﻿namespace Turbo.Plugins.LightningMod
{
    using Turbo.Plugins.Default;
    using SharpDX.DirectInput;
    using System.Linq;
    public class BarbarianWhirlwindPlugin : BasePlugin, IInGameTopPainter, IKeyEventHandler
    {
        public IKeyEvent ToggleKeyEvent { get; set; }
        public IFont InfoFont1 { get; private set; }
        public IFont InfoFont2 { get; private set; }
        public IFont InfoFont3 { get; private set; }
        private string str_running1;
        private string str_running2;
        private string str_tip;
        public bool Running { get; private set; }
        public bool Rotating { get; private set; }
        private bool isWhirlwindBarbarian = false;
        private IPlayerSkill skillWhirlwind = null;
        private IPlayerSkill skillAncientSpear = null;
        private IWatch DelayTimer;
        private IMonster Monster = null;

        public BarbarianWhirlwindPlugin()
        {
            Enabled = false;
        }
        public override void Load(IController hud)
        {
            base.Load(hud);
            ToggleKeyEvent = Hud.Input.CreateKeyEvent(true, Key.F3, false, false, false);
            InfoFont1 = Hud.Render.CreateFont("tahoma", 8, 255, 200, 200, 0, true, false, 255, 0, 0, 0, true);
            InfoFont2 = Hud.Render.CreateFont("tahoma", 8, 255, 0, 255, 0, true, false, 255, 0, 0, 0, true);
            InfoFont3 = Hud.Render.CreateFont("tahoma", 8, 255, 255, 0, 0, true, false, 255, 0, 0, 0, true);
            DelayTimer = Hud.Time.CreateWatch();
        }
        public void OnKeyEvent(IKeyEvent keyEvent)
        {
            if (ToggleKeyEvent.Matches(keyEvent) && keyEvent.IsPressed && isWhirlwindBarbarian)
            {
                if(Running)
                {
                    IWorldCoordinate coord = Hud.Window.CreateScreenCoordinate(Hud.Window.CursorX, Hud.Window.CursorY).ToWorldCoordinate();
                    IMonster _monster = Hud.Game.AliveMonsters.Where(m => m.FloorCoordinate.XYDistanceTo(coord) < 10 && m.Rarity != ActorRarity.RareMinion).OrderByDescending(m => m.MaxHealth).FirstOrDefault();
                    //IMonster _monster = Hud.Game.SelectedMonster2;
                    if(_monster != null &&_monster == Monster)
                    {//锁定同一个目标再次快捷键时停止锁定该目标
                        Monster = null;
                        Rotating = false;
                        DelayTimer.Stop();
                        return;
                    }
                    else
                    {//如果目标为空或切换新目标则刷新目标
                        Monster = _monster;
                    }

                    if(Rotating)
                    {
                        if(Monster == null)
                        {//锁定状态选中空地时停止锁定
                            Rotating = false;
                            DelayTimer.Stop();
                            return;
                        }
                    }
                    else
                    {
                        if (Monster == null)
                        {//非锁定目标状态选中空地时停止旋转
                            Running = false;
                            DelayTimer.Stop();
                            upWhirlwind();
                            return;
                        }
                        else
                        {//切换锁定目标
                            Rotating = true;
                            DelayTimer.Start();
                            return;
                        }
                    }
                }
                else
                {
                    Running = true;
                }

            }
        }

        private void upWhirlwind()
        {
            if (skillWhirlwind != null && Hud.Interaction.IsContinuousActionStarted(skillWhirlwind.Key))
            {
                Hud.Interaction.StopContinuousAction(skillWhirlwind.Key);//弹起旋风
            }
        }
        public void PaintTopInGame(ClipState clipState)
        {
            if (clipState != ClipState.AfterClip) return;
            if (Hud.CurrentLanguage == Language.zhCN)
            {
                str_running1 = "正在旋风，" + ToggleKeyEvent + "键围绕目标旋转或停止";
                str_running2 = "正在围绕目标旋转，" + ToggleKeyEvent + "键停止";
                str_tip = ToggleKeyEvent + "键开始旋风";
            }
            else if (Hud.CurrentLanguage == Language.zhTW)
            {
                str_running1 = "正在旋風，" + ToggleKeyEvent + "鍵圍繞目標旋轉或停止";
                str_running2 = "正在圍繞目標旋轉，" + ToggleKeyEvent + "鍵停止";
                str_tip = ToggleKeyEvent + "鍵開始旋風";
            }
            else
            {
                str_running1 = "Whirlwinding," + ToggleKeyEvent + "to rotating around the monster or stop";
                str_running2 = "Rotating around the monster," + ToggleKeyEvent + "to stop";
                str_tip = "Key to cast Whirlwind";
            }

            isWhirlwindBarbarian = Hud.Game.Me.HeroClassDefinition.HeroClass == HeroClass.Barbarian &&
               Hud.Game.Me.Powers.UsedSkills.Any(s => s.SnoPower == Hud.Sno.SnoPowers.Barbarian_Whirlwind)
                ;//获取自己是旋风蛮子
            if (isWhirlwindBarbarian)
            {
                skillWhirlwind = Hud.Game.Me.Powers.UsedBarbarianPowers.Whirlwind;
            }			
            if (Hud.Game.IsInTown)
            {
                if(Running)
                {
                    Running = false;
                    upWhirlwind();
                }
                return;
            }
            if (isWhirlwindBarbarian)
            {
                if (Running)
                {
                    if(Rotating)
                    {
                        InfoFont3.DrawText(str_running2, Hud.Game.Me.FloorCoordinate.ToScreenCoordinate().X - InfoFont3.GetTextLayout(str_running2).Metrics.Width / 2, Hud.Game.Me.FloorCoordinate.ToScreenCoordinate().Y, true);
                    }
                    else
                    {
                        InfoFont2.DrawText(str_running1, Hud.Game.Me.FloorCoordinate.ToScreenCoordinate().X - InfoFont2.GetTextLayout(str_running1).Metrics.Width / 2, Hud.Game.Me.FloorCoordinate.ToScreenCoordinate().Y, true);
                    }
                }
                else
                {
                    InfoFont1.DrawText(str_tip, Hud.Game.Me.FloorCoordinate.ToScreenCoordinate().X - InfoFont1.GetTextLayout(str_tip).Metrics.Width / 2, Hud.Game.Me.FloorCoordinate.ToScreenCoordinate().Y, true);
                }
            }
            else
            {
                return;
            }
            if (Running)
            {
                if(!Hud.Render.MinimapUiElement.Visible || Hud.Render.WorldMapUiElement.Visible || Hud.Game.IsPaused || Hud.Game.IsLoading || !Hud.Window.IsForeground)
                {
                    upWhirlwind();
                    return;
                }
                if (!Hud.Game.Me.IsDead && !Hud.Game.Me.Powers.BuffIsActive(Hud.Sno.SnoPowers.Generic_ActorGhostedBuff.Sno) && !Hud.Game.Me.Powers.BuffIsActive(Hud.Sno.SnoPowers.Barbarian_Whirlwind.Sno, 0))
                {
                    Hud.Interaction.StopContinuousAction(skillWhirlwind.Key);
                    Hud.Interaction.StartContinuousAction(skillWhirlwind.Key, skillWhirlwind.Key == ActionKey.LeftSkill);//按下旋风
                    Hud.Interaction.ShiftUp();
                }
                if (Rotating)
                {
                    if (Monster?.IsAlive == true && Monster.FloorCoordinate.XYDistanceTo(Hud.Game.Me.FloorCoordinate) < 200 && !Hud.Game.Me.IsDead && !Hud.Game.Me.Powers.BuffIsActive(Hud.Sno.SnoPowers.Generic_ActorGhostedBuff.Sno))//200码以内活着的目标且自己也活着
                    {
                        if (DelayTimer.ElapsedMilliseconds > 50 && !(Hud.Interaction.IsHotKeySet(ActionKey.Move) && Hud.Interaction.IsContinuousActionStarted(ActionKey.Move)))
                        {
                            Hud.Interaction.MouseMove(Monster.FloorCoordinate.ToScreenCoordinate().X, Monster.FloorCoordinate.ToScreenCoordinate().Y);//鼠标点到目标
                            DelayTimer.Restart();
                        }
                    }
                    else
                    {
                        Rotating = false;
                        Monster = null;
                        DelayTimer.Stop();
                    }
                }
            }
            skillAncientSpear = Hud.Game.Me.Powers.UsedBarbarianPowers.AncientSpear;//上古之矛
            IWorldCoordinate cursor = Hud.Window.CreateScreenCoordinate(Hud.Window.CursorX, Hud.Window.CursorY).ToWorldCoordinate();
			if (Hud.Game.Me.Powers.UsedSkills.Any(s => s.SnoPower == Hud.Sno.SnoPowers.Barbarian_AncientSpear) && skillAncientSpear != null && Hud.Game.Me.Stats.ResourcePctFury > 25 && cursor.XYDistanceTo(Hud.Game.Me.FloorCoordinate) > 40)// 使用矛技能，怒气大于25，鼠标离自己40码以上
			{
				IMonster SpearM = Hud.Game.AliveMonsters.Where(m => m.FloorCoordinate.XYDistanceTo(cursor) < 5).OrderBy(m => m.NormalizedXyDistanceToMe).FirstOrDefault();//离鼠标5码内最近的怪					
				if (SpearM != null)
				{
					string Juggernaut = "霸王";
					bool spear = true;
					foreach (var snoMonsterAffix in SpearM.AffixSnoList)
					{
						string affix = snoMonsterAffix.NameLocalized;
						if (affix == Juggernaut) //霸王不拉
						{
							spear = false;
						}
					}
					if (spear)
					{														
						Hud.Interaction.StopContinuousAction(skillWhirlwind.Key);
						Hud.Interaction.MouseMove(SpearM.FloorCoordinate.ToScreenCoordinate().X, SpearM.FloorCoordinate.ToScreenCoordinate().Y);//移动鼠标到怪身上
						Hud.Interaction.DoAction(skillAncientSpear.Key);
					}
				}
			}
			
        }
    }
}